@@ -7,8 +7,6 @@
 #' @examples
 #' library("polle")
 #' ### Simulating two-stage policy data
-#' source(system.file("sim", "two_stage.R", package="polle"))
-#' par0 <- c(gamma = 0.5, beta = 1)
 #' d <- sim_two_stage(2e3, seed=1, par=par0)
 #' pd <- policy_data(d,
 #'                   action = c("A_1", "A_2"),
@@ -28,27 +26,37 @@
 #' @noRd
 fit_g_function <- function(history, g_model){
   action_set <- getElement(history, "action_set")
+  stage_action_set <- getElement(history, "stage_action_set")
+  if(is.null(stage_action_set)){
+    stage_action_set <- action_set
+  }
+  stage <- getElement(history, "stage")
 
   # getting the action (A) and the model matrix (H):
   A <- get_A(history)
   H <- get_H(history)
-  stage <- unique(get_id_stage(history)[, "stage"])
 
-  # checking that all actions in the actions set occur:
-  if (!all(action_set == sort(unique(A)))){
-    mes <- "Not all actions occur at stage"
-    mes <- paste(mes, paste(stage, collapse = ", "))
-    mes <- paste(mes, ". Unable to fit g-function.", sep = "")
+  # checking that all actions in the stage action set occur:
+  if (!all(stage_action_set == sort(unique(A)))){
+    if (!is.null(stage)){
+      mes <- "Not all actions in the stage action set occur at stage"
+      mes <- paste(mes, paste(stage, collapse = ", "))
+      mes <- paste(mes, ". Unable to fit g-function.", sep = "")
+    } else{
+      mes <- "Not all actions in the action set occur. Unable to fit g-function."
+    }
     stop(mes)
   }
 
   # fitting the model:
-  g_model <- g_model(A = A, H = H, action_set = action_set)
+  g_model <- g_model(A = A, H = H, action_set = stage_action_set)
 
   g_function <- list(
     g_model = g_model,
     H_names = colnames(H),
-    action_set = action_set
+    action_set = action_set,
+    stage_action_set = stage_action_set,
+    stage = stage
   )
   class(g_function) <- "g_function"
 
@@ -64,27 +72,62 @@ print.g_function <- function(x, ...){
   print(y)
 }
 
-evaluate.g_function <- function(object, new_history){
-  g_model <- object$g_model
-  H_names <- object$H_names
-  action_set <- object$action_set
+predict.g_function <- function(object, new_history){
+  g_model <- getElement(object, "g_model")
+  H_names <- getElement(object, "H_names")
+  # action set of the new history object:
+  action_set <- getElement(new_history, "action_set")
+  # stage action set of the fitted g-function:
+  stage_action_set <- getElement(object, "stage_action_set")
+  stage <- getElement(object, "stage")
 
   id_stage <- get_id_stage(new_history)
   new_H <- get_H(new_history)
 
+  # checks
+  if(!all(stage_action_set %in% action_set))
+    stop("The fitted stage action set is not a subset of the new action set.")
   if (!all(names(new_H) %in% H_names))
     stop("new_history does not have the same column names as the original history.")
 
   g_values <- predict(g_model, new_H = new_H)
+  if (!all(g_values >= 0) |
+      !all(g_values <= 1)){
+    stop("Not all g-model values are within [0,1].")
+  }
+
+  tmp <- matrix(data = 0, nrow = nrow(id_stage), ncol = length(action_set))
+  colnames(tmp) <- action_set
+  tmp[, stage_action_set] <- g_values
+  g_values <- tmp; rm(tmp)
   colnames(g_values) <- paste("g", action_set, sep = "_")
 
   if (!all(complete.cases(g_values))){
-    stage <- unique(new_history$H$stage)
-    mes <- paste("Evaluation of the g-function at stage ", stage, " have missing values.", sep = "")
+    if(!is.null(stage)){
+      mes <- paste("The g-function predictions at stage ",
+                   stage,
+                   " have missing values.",
+                   sep = "")
+    } else {
+      mes <- "The g-function predictions have missing values."
+    }
     stop(mes)
   }
 
-  # including the id's
+  if (!all(abs(apply(g_values, sum, MARGIN = 1) - 1) < 1e-12)){
+    if(!is.null(stage)){
+      mes <- paste("The g-function predictions at stage ",
+                   stage,
+                   " do not sum to 1.",
+                   sep = "")
+    } else {
+      mes <- "The g-function predictions do not sum to 1."
+    }
+    stop(mes)
+  }
+
+
+  # including the id's and stage number(s)
   g_values <- data.table(id_stage, g_values)
   setkeyv(g_values, c("id", "stage"))
 
@@ -93,16 +136,16 @@ evaluate.g_function <- function(object, new_history){
 
 #' Fit g-functions
 #'
-#' \code{fit_g_functions} is used to fit a list of g-models
+#' \code{fit_g_functions} is used to fit a list of g-models.
 #' @param policy_data Policy data object created by [policy_data()].
-#' @param g_models Propensity models/g-models created by [g_glm()], [g_rf()], [g_sl()] or similar functions.
-#' @param full_history If TRUE, the full history is used to fit each g-model. If FALSE, the single stage/"Markov type" history is used to fit each g-model.
+#' @param g_models List of action probability models/g-models for each stage
+#' created by [g_empir()], [g_glm()], [g_rf()], [g_sl()] or similar functions.
+#' @param full_history If TRUE, the full history is used to fit each g-model.
+#' If FALSE, the single stage/"Markov type" history is used to fit each g-model.
 #' @examples
 #' library("polle")
 #' ### Simulating two-stage policy data
-#' source(system.file("sim", "two_stage.R", package="polle"))
-#' par0 <- c(gamma = 0.5, beta = 1)
-#' d <- sim_two_stage(2e3, seed=1, par=par0)
+#' d <- sim_two_stage(2e3, seed=1)
 #' pd <- policy_data(d,
 #'                   action = c("A_1", "A_2"),
 #'                   covariates = list(L = c("L_1", "L_2"),
@@ -121,30 +164,48 @@ evaluate.g_function <- function(object, new_history){
 #'                                g_models = list(g_glm(), g_glm()),
 #'                                full_history = TRUE)
 #' g_functions
-#' @noRd
-fit_g_functions <- function(policy_data, g_models, full_history){
+#' @export
+fit_g_functions <- function(policy_data, g_models, full_history = FALSE){
   K <- get_K(policy_data)
 
+  # input checks:
+  if (!(is.logical(full_history) & (length(full_history) == 1)))
+    stop("full_history must be TRUE or FALSE")
   if (is.null(g_models))
     stop("Please provide g_models.")
-
-  # checking the g_models:
-  if (class(g_models)[[1]] == "list"){
-    if (length(g_models) != K) stop("g_models must either be a list of length K or a single g-model.")
+  mes <- "g_models must be a single g_model or a list of K g_models's."
+  if (is.list(g_models)){
+    tmp <- all(unlist(lapply(g_models, function(gm) inherits(gm, "g_model"))))
+    if (!tmp)
+      stop(mes)
+    rm(tmp)
+    if (length(g_models) != K)
+      stop(mes)
   } else{
-    if (full_history == TRUE) stop("full_history must be FALSE when a single g-model is provided.")
+    if (!inherits(g_models, "g_model"))
+      stop(mes)
+    if (full_history == TRUE)
+      stop("full_history must be FALSE when a single g-model is provided.")
   }
 
-  if (class(g_models)[[1]] == "list"){
-    history <- lapply(1:K, function(s) get_history(policy_data, stage = s, full_history = full_history))
-    g_functions <- mapply(history, g_models, FUN = function(h, gm) fit_g_function(history = h, g_model = gm), SIMPLIFY = FALSE)
+  # fitting the g-models:
+  if (is.list(g_models)){
+    history <- lapply(1:K,
+                      function(s) get_history(policy_data,
+                                              stage = s,
+                                              full_history = full_history))
+    g_functions <- mapply(history,
+                          g_models,
+                          FUN = function(h, gm) fit_g_function(history = h,
+                                                               g_model = gm),
+                          SIMPLIFY = FALSE)
     names(g_functions) <- paste("stage_", 1:K, sep = "")
   } else{
     history <- state_history(policy_data)
     g_functions <- list(all_stages = fit_g_function(history, g_models))
   }
 
-  class(g_functions) <- "nuisance_functions"
+  class(g_functions) <- c("g_functions", "nuisance_functions")
   attr(g_functions, "full_history") <- full_history
 
   return(g_functions)
@@ -162,7 +223,6 @@ fit_g_functions <- function(policy_data, g_models, full_history){
 #' @examples
 #' #' library("polle")
 #' ### Two stage:
-#' source(system.file("sim", "two_stage.R", package="polle"))
 #' d <- sim_two_stage(2e3, seed=1)
 #' pd <- policy_data(d,
 #'                   action = c("A_1", "A_2"),
@@ -201,13 +261,13 @@ fit_g_functions_cf <- function(folds,
   future_args <- append(future_args, list(X = folds,
                                           FUN = function(f){
                                             train_id <- id[-f]
-                                            train_policy_data <- subset(policy_data, train_id)
+                                            train_policy_data <- subset_id(policy_data, train_id)
                                             if (train_policy_data$dim$K != K) stop("The number of stages K varies across the training policy data folds.")
                                             train_g_functions <- fit_g_functions(policy_data = train_policy_data, g_models = g_models, full_history = full_history)
 
                                             valid_id <- id[f]
-                                            valid_policy_data <- subset(policy_data, valid_id)
-                                            valid_g_values <- evaluate(train_g_functions, valid_policy_data)
+                                            valid_policy_data <- subset_id(policy_data, valid_id)
+                                            valid_g_values <- predict(train_g_functions, valid_policy_data)
 
                                             list(
                                               train_g_functions = train_g_functions,
@@ -240,34 +300,30 @@ fit_g_functions_cf <- function(folds,
 #' @seealso [predict.nuisance_functions]
 #' @examples
 #' ### Two stages:
-#' source(system.file("sim", "two_stage.R", package="polle"))
-#' d2 <- sim_two_stage(5e2, seed=1)
-#' pd2 <- policy_data(d2,
+#' d <- sim_two_stage(5e2, seed=1)
+#' pd <- policy_data(d,
 #'                   action = c("A_1", "A_2"),
 #'                   baseline = c("B"),
 #'                   covariates = list(L = c("L_1", "L_2"),
 #'                                     C = c("C_1", "C_2")),
 #'                   utility = c("U_1", "U_2", "U_3"))
-#' pd2
+#' pd
 #'
 #' # evaluating the static policy a=1 using inverse propensity weighting
 #' # based on a GLM model at each stage
-#' pe2 <- policy_eval(type = "ipw",
-#'                    policy_data = pd2,
-#'                    policy = policy_def(1, reuse = TRUE, name = "A=1"),
-#'                    g_models = list(g_glm(), g_glm()))
-#' pe2
+#' pe <- policy_eval(type = "ipw",
+#'                   policy_data = pd,
+#'                   policy = policy_def(1, reuse = TRUE, name = "A=1"),
+#'                   g_models = list(g_glm(), g_glm()))
+#' pe
 #'
 #' # getting the g-functions
-#' g_functions <- get_g_functions(pe2)
+#' g_functions <- get_g_functions(pe)
 #' g_functions
 #'
 #' # getting the fitted g-function values
-#' head(predict(g_functions, pd2))
+#' head(predict(g_functions, pd))
 #' @export
 get_g_functions <- function(object)
   UseMethod("get_g_functions")
 
-
-
-
