@@ -1,11 +1,12 @@
 #' Policy Evaluation
 #'
-#' \code{policy_eval()} is used to estimate the value of a given fixed policy or a data adaptive policy (e.g. a policy learned from the data).
+#' \code{policy_eval()} is used to estimate the value of a given fixed policy
+#' or a data adaptive policy (e.g. a policy learned from the data).
 #' @param policy_data Policy data object created by [policy_data()].
 #' @param policy Policy object created by [policy_def()].
 #' @param policy_learn Policy learner object created by [policy_learn()].
 #' @param g_models List of action probability models/g-models for each stage
-#' created by [g_glm()], [g_rf()], [g_sl()] or similar functions.
+#' created by [g_empir()], [g_glm()], [g_rf()], [g_sl()] or similar functions.
 #' Only used for evaluation if \code{g_functions} is \code{NULL}.
 #' If a single model is provided and \code{g_full_history} is \code{FALSE},
 #' a single g-model is fitted across all stages. If \code{g_full_history} is
@@ -23,7 +24,8 @@
 #' If FALSE, the state/Markov type history is used to fit each g-model.
 #' @param q_full_history Similar to g_full_history.
 #' @param M Number of folds for cross-fitting.
-#' @param type Type of evaluation (dr/doubly robust, ipw/inverse propensity weighting, or/outcome regression).
+#' @param type Type of evaluation (dr/doubly robust, ipw/inverse propensity
+#' weighting, or/outcome regression).
 #' @param future_args Arguments passed to [future.apply::future_apply()].
 #' @param name Character string.
 #' @param object,x,y Objects of class "policy_eval".
@@ -34,8 +36,10 @@
 #' @returns \code{policy_eval()} returns an object of class "policy_eval".
 #' The object is a list containing the following elements:
 #' \item{\code{value_estimate}}{Numeric. The estimated value of the policy.}
-#' \item{\code{type}}{Character string. The type of evaluation ("dr", "ipw", "or").}
-#' \item{\code{IC}}{Numeric vector. Estimated influence curve associated with the value estimate.}
+#' \item{\code{type}}{Character string. The type of evaluation ("dr", "ipw",
+#' "or").}
+#' \item{\code{IC}}{Numeric vector. Estimated influence curve associated with
+#' the value estimate.}
 #' \item{\code{value_estimate_ipw}}{(only if \code{type = "dr"}) Numeric.
 #' The estimated value of the policy based on inverse probability weighting.}
 #' \item{\code{value_estimate_or}}{(only if \code{type = "dr"}) Numeric.
@@ -55,7 +59,7 @@
 #' for cross-fitting.}
 #' @section S3 generics:
 #' The following S3 generic functions are available for an object of
-#' class \code{policy_data}:
+#' class \code{policy_eval}:
 #' \itemize{
 #' \item{[get_g_functions()]}{ Extract the fitted g-functions.}
 #' \item{[get_q_functions()]}{ Extract the fitted Q-functions.}
@@ -63,6 +67,7 @@
 #' \item{[get_policy_functions()]}{ Extract the fitted policy function for
 #'                                 a given stage.}
 #' \item{[get_policy_actions()]}{ Extract the (fitted) policy actions.}
+#' \item{[plot.policy_eval()]}{Plot diagnostics.}
 #' }
 #' @seealso [lava::IC], [lava::estimate.default].
 #' @details
@@ -130,7 +135,6 @@
 #' @examples
 #' library("polle")
 #' ### Single stage:
-#' source(system.file("sim", "single_stage.R", package="polle"))
 #' d1 <- sim_single_stage(5e2, seed=1)
 #' pd1 <- policy_data(d1, action="A", covariates=list("Z", "B", "L"), utility="U")
 #' pd1
@@ -172,7 +176,6 @@
 #' head(IC(est1))
 #'
 #' ### Two stages:
-#' source(system.file("sim", "two_stage.R", package="polle"))
 #' d2 <- sim_two_stage(5e2, seed=1)
 #' pd2 <- policy_data(d2,
 #'                    action = c("A_1", "A_2"),
@@ -182,8 +185,8 @@
 #' pd2
 #'
 #' # defining a policy learner based on cross-fitted doubly robust Q-learning:
-#' pl2 <- policy_learn(type = "rqvl",
-#'                     control = control_rqvl(qv_models = list(q_glm(~C_1),
+#' pl2 <- policy_learn(type = "drql",
+#'                     control = control_drql(qv_models = list(q_glm(~C_1),
 #'                                                             q_glm(~C_1+C_2))),
 #'                     full_history = TRUE,
 #'                     L = 2) # number of folds for cross-fitting
@@ -195,7 +198,7 @@
 #'                    q_models = q_glm(),
 #'                    g_models = g_glm(),
 #'                    M = 2, # number of folds for cross-fitting
-#'                    name = "rqvl")
+#'                    name = "drql")
 #' # summarizing the estimated value of the policy:
 #' pe2
 #'
@@ -203,10 +206,10 @@
 #' head(get_policy_actions(pe2))
 policy_eval <- function(policy_data,
                         policy = NULL, policy_learn = NULL,
-                        g_functions=NULL, g_models=g_glm(), g_full_history = FALSE,
-                        q_functions=NULL, q_models=q_glm(), q_full_history = FALSE,
+                        g_functions = NULL, g_models = g_glm(), g_full_history = FALSE,
+                        q_functions = NULL, q_models = q_glm(), q_full_history = FALSE,
                         type = "dr",
-                        M=1, future_args = list(future.seed = TRUE),
+                        M = 1, future_args = list(future.seed=TRUE),
                         name = NULL
                         ) {
   args <- as.list(environment())
@@ -215,6 +218,47 @@ policy_eval <- function(policy_data,
   args[["future_args"]] <- NULL
   args[["name"]] <- NULL
 
+  # input checks:
+  if (!inherits(policy_data, what = "policy_data"))
+    stop("policy_data must be of inherited class 'policy_data'.")
+  if (!is.null(policy)){
+    if (!inherits(policy, what = "policy"))
+      stop("policy must be of inherited class 'policy'.")
+  }
+  if ((is.null(policy) & is.null(policy_learn)) |
+      (!is.null(policy_learn) & !is.null(policy)))
+    stop("Provide either policy or policy_learn.")
+  if (is.null(policy) & !is.null(policy_learn)){
+    if (!inherits(policy_learn, what = "policy_learn"))
+      stop("policy_learn must be of inherited class 'policy_learn'.")
+  }
+  if (!is.null(g_functions)){
+    if(!(inherits(g_functions, "g_functions")))
+      stop("g_functions must be of class 'g_functions'.")
+  }
+  if (!(is.logical(g_full_history) & (length(g_full_history) == 1)))
+    stop("g_full_history must be TRUE or FALSE")
+  if (!is.null(q_functions)){
+    if(!(inherits(q_functions, "q_functions")))
+      stop("q-functions must be of class 'q_functions'.")
+  }
+  if (!(is.logical(q_full_history) & (length(q_full_history) == 1)))
+    stop("q_full_history must be TRUE or FALSE")
+  if (!(is.numeric(M) & (length(M) == 1)))
+    stop("M must be an integer greater than 0.")
+  if (!(M %% 1 == 0))
+    stop("M must be an integer greater than 0.")
+  if (M<=0)
+    stop("M must be an integer greater than 0.")
+  if (!is.list(future_args))
+    stop("future_args must be a list.")
+  if (!is.null(name)){
+    name <- as.character(name)
+    if (length(name) != 1)
+      stop("name must be a character string.")
+  }
+
+
   if (M > 1){
     val <- policy_eval_cross(args = args,
                                     policy_data = policy_data,
@@ -270,6 +314,10 @@ policy_eval_type <- function(type,
   }
   policy_actions <- policy(valid_policy_data)
 
+  # checking that the (fitted) policy actions comply with the stage action sets:
+  check_actions(actions = policy_actions,
+                policy_data = train_policy_data)
+
   # calculating the doubly robust score and value estimate:
   value_object <- value(type = type,
                         policy_data = valid_policy_data,
@@ -306,10 +354,12 @@ policy_eval_cross <- function(args,
   folds <- lapply(folds, sort)
   names(folds) <- paste("fold_", 1:M, sep = "")
 
+  prog <- progressor(along = folds)
   cross_args <- append(list(X = folds,
                             FUN = policy_eval_fold,
                             policy_data = policy_data,
-                            args = args),
+                            args = args,
+                            prog = prog),
                        future_args)
 
   # cross fitting the policy evaluation using the folds:
@@ -366,7 +416,8 @@ policy_eval_cross <- function(args,
 
 policy_eval_fold <- function(fold,
                              policy_data,
-                             args
+                             args,
+                             prog
 ){
 
   K <- get_K(policy_data)
@@ -376,11 +427,11 @@ policy_eval_fold <- function(fold,
   validation_id <- id[fold]
 
   # training data:
-  train_policy_data <- subset(policy_data, train_id)
+  train_policy_data <- subset_id(policy_data, train_id)
   if (get_K(train_policy_data) != K) stop("The number of stages varies accross the training folds.")
 
   # validation data:
-  valid_policy_data <- subset(policy_data, validation_id)
+  valid_policy_data <- subset_id(policy_data, validation_id)
   if (get_K(valid_policy_data) != K) stop("The number of stages varies accross the validation folds.")
 
   eval_args <- append(args, list(valid_policy_data = valid_policy_data,
@@ -388,154 +439,10 @@ policy_eval_fold <- function(fold,
 
   out <- do.call(what = "policy_eval_type", args = eval_args)
 
+  # progress:
+  prog()
+
   return(out)
 }
 
-#' @rdname policy_eval
-#' @export
-coef.policy_eval <- function(object, ...) {
-  return(object$value_estimate)
-}
-
-#' @rdname policy_eval
-#' @export
-IC.policy_eval <- function(x, ...) {
-  res <- cbind(getElement(x, "IC"))
-  return(res)
-}
-
-#' @rdname policy_eval
-#' @export
-vcov.policy_eval <- function(object, ...) {
-  ic <- IC(object)
-  n <- nrow(ic)
-  return(crossprod(ic)/(n*n))
-}
-
-#' @rdname policy_eval
-#' @export
-print.policy_eval <- function(x, ...) {
-  print(summary(x, ...))
-}
-
-
-#' @rdname policy_eval
-#' @export
-summary.policy_eval <- function(object, ...) {
-  lava::estimate(object, ...)
-}
-
-#' @rdname policy_eval
-#' @export
-estimate.policy_eval <- function(x, ..., labels=x$name) {
-  p <- length(coef(x))
-  if (is.null(labels)) {
-    if (p==1) {
-      "value"
-    } else {
-      labels <- paste0("value", seq(p))
-    }
-  }
-  est <- lava::estimate(NULL, coef=coef(x), IC=IC(x), labels=labels, ...)
-  return(est)
-}
-
-#' @rdname policy_eval
-#' @export
-"merge.policy_eval" <- function(x, y, ..., paired = TRUE) {
-  dots <- list(...)
-  idx <- names(dots) %in% formalArgs(lava::estimate.default)[-1]
-  est_args <- list()
-  if (length(idx)>0) {
-    est_args <- dots[which(idx)]
-    dots <- dots[-which(idx)]
-  }
-  m <- lapply(c(list(x, y), dots), function(p)
-    do.call(estimate, c(list(p),est_args)))
-  m <- do.call("merge", c(m, list(paired=paired)))
-  return(m)
-}
-
-#' @rdname policy_eval
-#' @export
-"+.policy_eval" <- function(x,...) {
-  merge(x, ...)
-}
-
-#' @export
-get_g_functions.policy_eval <- function(object){
-  getElement(object, "g_functions")
-}
-
-#' @export
-get_q_functions.policy_eval <- function(object){
-  getElement(object, "q_functions")
-}
-
-#' @export
-get_policy.policy_eval <- function(object){
-  po <- getElement(object, "policy_object")
-  if (is.null(po)){
-    mes <- "Learned policy is not available."
-    stop(mes)
-  }
-  pf <- get_policy(po)
-  return(pf)
-}
-
-#' @title Get Policy Actions
-#'
-#' @description \code{get_policy_actions()} extract the actions dictated by the
-#' (learned and possibly cross-fitted) policy a every stage.
-#' @param object Object of class [policy_eval].
-#' @returns [data.table] with keys \code{id} and \code{stage} and action variable
-#' \code{d}.
-#' @examples
-#' ### Two stages:
-#' source(system.file("sim", "two_stage.R", package="polle"))
-#' d2 <- sim_two_stage(5e2, seed=1)
-#' pd2 <- policy_data(d2,
-#'                   action = c("A_1", "A_2"),
-#'                   covariates = list(L = c("L_1", "L_2"),
-#'                                     C = c("C_1", "C_2")),
-#'                   utility = c("U_1", "U_2", "U_3"))
-#' pd2
-#'
-#' # defining a policy learner based on cross-fitted doubly robust Q-learning:
-#' pl2 <- policy_learn(type = "rqvl",
-#'                    control = control_rqvl(qv_models = list(q_glm(~C_1), q_glm(~C_1+C_2))),
-#'                    full_history = TRUE,
-#'                    L = 2) # number of folds for cross-fitting
-#'
-#' # evaluating the policy learner using 2-fold cross fitting:
-#' pe2 <- policy_eval(type = "dr",
-#'                    policy_data = pd2,
-#'                    policy_learn = pl2,
-#'                    q_models = q_glm(),
-#'                    g_models = g_glm(),
-#'                    M = 2) # number of folds for cross-fitting
-#'
-#' # Getting the cross-fitted actions dictated by the fitted policy:
-#' head(get_policy_actions(pe2))
-#'
-#' @export
-get_policy_actions <- function(object)
-  UseMethod("get_policy_actions")
-
-#' @export
-get_policy_actions.policy_eval <- function(object){
-  getElement(object, "policy_actions")
-}
-
-#' @export
-get_policy_functions.policy_eval <- function(object, stage){
-  po <- getElement(object, "policy_object")
-  if (is.null(po)){
-    mes <- "Learned policy is not available."
-    stop(mes)
-  }
-  pf <- get_policy_functions(po, stage = stage)
-  return(pf)
-}
-
 
