@@ -3,23 +3,20 @@ rql<- function(policy_data, alpha,
                q_models, q_full_history,
                L,
                ...){
-  K <- policy_data$dim$K
-  n <- policy_data$dim$n
-  action_set <- policy_data$action_set
-
-  if (!(is.numeric(alpha) &  (length(alpha) == 1) & (alpha >=0 & alpha < 0.5)))
-    stop("alpha must be numeric and in [0, 0.5).")
+  K <- get_K(policy_data)
+  n <- get_n(policy_data)
+  action_set <- get_action_set(policy_data)
 
+  # input checks:
   if (L != 1)
-    stop("L must be 1 when using rql (no cross-fitting).")
-
+    stop("L must be 1 when type = 'ql' (no cross-fitting).")
   if (!is.null(g_functions)){
-    if(!(class(g_functions)[[1]] == "nuisance_functions"))
-      stop("g-functions must be of class 'nuisance_functions'.")
+    if(!inherits(g_functions, what = "g_functions"))
+      stop("g-functions must be of class 'g_functions'.")
   }
-
-  if (class(q_models)[[1]] == "list"){
-    if (length(q_models) != K) stop("q_models must either be a list of length K or a single Q-model.")
+  if (is.list(q_models)){
+    if (length(q_models) != K)
+      stop("q_models must either be a list of length K or a single Q-model.")
   }
 
   # getting the IDs:
@@ -36,7 +33,7 @@ rql<- function(policy_data, alpha,
     if (is.null(g_functions)){
       g_functions <- fit_g_functions(policy_data, g_models = g_models, full_history = g_full_history)
     }
-    g_values <- evaluate(g_functions, policy_data)
+    g_values <- predict(g_functions, policy_data)
   }
 
   # (n X K+1) matrix with entries Q_k(H_{k,i}, d_k(H_{k,i})), Q_{K+1} = U:
@@ -81,11 +78,10 @@ rql<- function(policy_data, alpha,
     Q[idx_k, k] <- q_max_realistic_k
     Q[!idx_k, k] <- Q[!idx_k, k+1]
   }
+  names(q_functions) <- paste("stage_", 1:K, sep = "")
   class(q_functions) <- "nuisance_functions"
   attr(q_functions, "full_history") <- q_full_history
 
-  # Zd_or <- Q[, 1]
-
   out <- list(
     q_functions = q_functions,
     g_functions = g_functions,
@@ -94,13 +90,13 @@ rql<- function(policy_data, alpha,
     K = K
   )
   out <- remove_null_elements(out)
-  class(out) <- c("RQL", "policy_object", "list")
+  class(out) <- c("ql", "policy_object", "list")
 
   return(out)
 }
 
 #' @export
-get_policy.RQL <- function(object){
+get_policy.ql <- function(object){
   g_functions <- get_g_functions(object)
   q_functions <- get_q_functions(object)
   action_set <- getElement(object, "action_set")
@@ -112,11 +108,11 @@ get_policy.RQL <- function(object){
 
   policy <- function(policy_data){
     # evaluating the Q-functions:
-    q_values <- evaluate(q_functions, policy_data = policy_data)
+    q_values <- predict(q_functions, policy_data)
 
     if (alpha != 0){
       # evaluating the g-functions:
-      g_values <- evaluate(g_functions, policy_data = policy_data)
+      g_values <- predict(g_functions, policy_data)
       # calculating the realistic actions:
       realistic_actions <- t(apply(
         g_values[ , g_cols, with = FALSE],
@@ -148,6 +144,9 @@ get_policy.RQL <- function(object){
 
     return(policy_actions)
   }
-  class(policy) <- c("policy", "function")
+
+  # setting class and attributes:
+  policy <- new_policy(policy, name = "ql")
+
   return(policy)
 }
